// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

import 'package:deequery/deequery.dart';
import 'package:darmatch/matchers.dart';

class Person {
  String name;
  int age;

  Person(this.name, this.age);

  String toString() => "$name ($age years)";
}

class Employee {
  String name, department;
  num grade;

  Employee(this.name, this.department, this.grade);

  String toString() => "$name (department: $department, grade: $grade)";
}

main() {
  var c = [-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10];

  var q1 = query(c).filter(even()).reject(lessThan(0)).map( (i) => i * 2 ).reverse().skip(1).take(2);
  for (final e in q1) {
    print(e);
  }

  print('--');

  var q2 = query(c).filter( (i) => i < 0 ).sort( (a, b) => b - a ).findFirst(odd());
  print(q2);

  print('--');

  var q3 = query(c).filter(positive()).fold(0, (acc, i) => acc + i );
  print(q3);

  print('--');

  var q4 = query(c).filter(positive()).map( (i) => i * -2 ).min();
  print(q4);

  print('--');

  var people = [
    new Person("Albert Einstein", 76),
    new Person("Ladislav Thon", 29),
    new Person("Test Person", 56),
    new Person("Yoda", 900)
  ];

  var q5 = query(people).sortBy( (p) => p.age );
  for (final e in q5) {
    print(e);
  }

  print('--');

  var employees = [
    new Employee("John", "Marketing", 2.4),
    new Employee("Mary", "Marketing", 2.4),
    new Employee("Gerald", "Sales", 3.6),
    new Employee("Antony", "Administration", 4.2),
    new Employee("Peter", "Administration", 4.8)
  ];

  var q6 = query(employees).sortBy( (e) => [e.department, desc(e.grade), e.name] );
  for (final e in q6) {
    print(e);
  }

  print('--');

  var q7 = query([1, 2, 3, 4, 5]).append([1, 3, 5, 7, 9]).append([2, 4, 6, 8]).unique();
  for (final e in q7) {
    print(e);
  }
}
