// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

part of deequery;

class _InvertingComparable implements Comparable {
  Comparable _delegate;

  _InvertingComparable(this._delegate);

  int compareTo(Comparable other) {
    checkThat(other, new IsType<_InvertingComparable>(),
        'Can only compare with another _InvertingComparable');
    return -1 * _delegate.compareTo(other._delegate);
  }
}

class _CompoundComparable implements Comparable {
  List<Comparable> delegates;

  _CompoundComparable(this.delegates);

  int compareTo(Comparable other) {
    checkThat(other, new IsType<_CompoundComparable>(),
        'Can only compare with another _CompoundComparable');
    checkThat(other.delegates, collectionLength(equals(delegates.length)),
        'Both _CompoundComparables must have same length');

    for (int i = 0; i < delegates.length; i++) {
      var comparison = delegates[i].compareTo(other.delegates[i]);
      if (comparison != 0) {
        return comparison;
      }
    }
    return 0;
  }
}

/// Returns the given [Comparable] as is. Useful for `Query.sortBy`.
Comparable asc(Comparable c) => c;
/// Returns a [Comparable] that is an inversion of [c].
/// Useful for `Query.sortBy`.
Comparable desc(Comparable c) => new _InvertingComparable(c);
